@@ -52,6 +52,7 @@ import pprint
 import copy
 
 from django.contrib.auth.models import User, Group, Permission
+import django.conf
 from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 import django.dispatch
@@ -86,10 +87,16 @@ from django_auth_ldap.config import _LDAPConfig, LDAPSearch
 logger = _LDAPConfig.get_logger()
 
 
-# Signals for populating user objects.
+# Exported signals
+
+# Allows clients to perform custom user/profile population.
 populate_user = django.dispatch.Signal(providing_args=["user", "ldap_user"])
 populate_user_profile = django.dispatch.Signal(providing_args=["profile", "ldap_user"])
 
+# Allows clients to inspect and perform special handling of LDAPError
+# exceptions. Exceptions raised by handlers will be propagated out.
+ldap_error = django.dispatch.Signal(providing_args=['context', 'exception'])
+
 
 class LDAPBackend(object):
     """
@@ -108,6 +115,9 @@ class LDAPBackend(object):
     # support multiple collections of settings.
     settings_prefix = 'AUTH_LDAP_'
 
+    # Default settings to override the built-in defaults.
+    default_settings = {}
+
     def __getstate__(self):
         """
         Exclude certain cached properties from pickling.
@@ -117,7 +127,8 @@ class LDAPBackend(object):
 
     def _get_settings(self):
         if self._settings is None:
-            self._settings = LDAPSettings(self.settings_prefix)
+            self._settings = LDAPSettings(self.settings_prefix,
+                                          self.default_settings)
 
         return self._settings
 
@@ -128,9 +139,7 @@ class LDAPBackend(object):
 
     def _get_ldap(self):
         if self._ldap is None:
-            from django.conf import settings
-
-            options = getattr(settings, 'AUTH_LDAP_GLOBAL_OPTIONS', None)
+            options = getattr(django.conf.settings, 'AUTH_LDAP_GLOBAL_OPTIONS', None)
 
             self._ldap = _LDAPConfig.get_ldap(options)
 
@@ -187,7 +196,7 @@ class LDAPBackend(object):
         if not hasattr(user, 'ldap_user') and self.settings.AUTHORIZE_ALL_USERS:
             _LDAPUser(self, user=user)  # This sets user.ldap_user
 
-        if hasattr(user, 'ldap_user'):
+        if hasattr(user, 'ldap_user') and (user.ldap_user.dn is not None):
             return user.ldap_user.get_group_permissions()
         else:
             return set()
@@ -333,8 +342,11 @@ class _LDAPUser(object):
         except self.AuthenticationFailed as e:
             logger.debug(u"Authentication failed for %s: %s" % (self._username, e))
         except ldap.LDAPError as e:
-            logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                           self._username, pprint.pformat(e))
+            results = ldap_error.send(self.backend.__class__,
+                                      context='authenticate', exception=e)
+            if len(results) == 0:
+                logger.warning(u"Caught LDAPError while authenticating %s: %s",
+                               self._username, pprint.pformat(e))
         except Exception:
             logger.exception(u"Caught Exception while authenticating %s",
                              self._username)
@@ -354,8 +366,12 @@ class _LDAPUser(object):
                 try:
                     self._load_group_permissions()
                 except ldap.LDAPError as e:
-                    logger.warning("Caught LDAPError loading group permissions: %s",
-                                   pprint.pformat(e))
+                    results = ldap_error.send(self.backend.__class__,
+                                              context='get_group_permissions',
+                                              exception=e)
+                    if len(results) == 0:
+                        logger.warning("Caught LDAPError loading group permissions: %s",
+                                       pprint.pformat(e))
 
         return self._group_permissions
 
@@ -373,8 +389,11 @@ class _LDAPUser(object):
 
             user = self._user
         except ldap.LDAPError as e:
-            logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                           self._username, pprint.pformat(e))
+            results = ldap_error.send(self.backend.__class__,
+                                      context='populate_user', exception=e)
+            if len(results) == 0:
+                logger.warning(u"Caught LDAPError while authenticating %s: %s",
+                               self._username, pprint.pformat(e))
         except Exception as e:
             logger.error(u"Caught Exception while authenticating %s: %s",
                          self._username, pprint.pformat(e))
@@ -558,7 +577,7 @@ class _LDAPUser(object):
         # We populate the profile after the user model is saved to give the
         # client a chance to create the profile. Custom user models in Django
         # 1.5 probably won't have a get_profile method.
-        if should_populate and django.VERSION < (1, 7) and hasattr(self._user, 'get_profile'):
+        if should_populate and self._should_populate_profile():
             self._populate_and_save_user_profile()
 
     def _populate_user(self):
@@ -582,6 +601,11 @@ class _LDAPUser(object):
             value = any(self._get_groups().is_member_of(dn) for dn in group_dns)
             setattr(self._user, field, value)
 
+    def _should_populate_profile(self):
+        return ((django.VERSION < (1, 7)) and
+                (getattr(django.conf.settings, 'AUTH_PROFILE_MODULE', None) is not None) and
+                hasattr(self._user, 'get_profile'))
+
     def _populate_and_save_user_profile(self):
         """
         Populates a User profile object with fields from the LDAP directory.
@@ -642,11 +666,17 @@ class _LDAPUser(object):
         Mirrors the user's LDAP groups in the Django database and updates the
         user's membership.
         """
-        group_names = self._get_groups().get_group_names()
-        groups = [Group.objects.get_or_create(name=group_name)[0] for group_name
-                  in group_names]
+        target_group_names = frozenset(self._get_groups().get_group_names())
+        current_group_names = frozenset(self._user.groups.values_list('name', flat=True).iterator())
 
-        self._user.groups = groups
+        if target_group_names != current_group_names:
+            existing_groups = list(Group.objects.filter(name__in=target_group_names).iterator())
+            existing_group_names = frozenset(group.name for group in existing_groups)
+
+            new_groups = [Group.objects.get_or_create(name=name)[0] for name
+                          in target_group_names if name not in existing_group_names]
+
+            self._user.groups = existing_groups + new_groups
 
     #
     # Group information
@@ -866,13 +896,13 @@ class LDAPSettings(object):
         'USER_SEARCH': None,
     }
 
-    def __init__(self, prefix='AUTH_LDAP_'):
+    def __init__(self, prefix='AUTH_LDAP_', defaults={}):
         """
         Loads our settings from django.conf.settings, applying defaults for any
         that are omitted.
         """
-        from django.conf import settings
+        defaults = dict(self.defaults, **defaults)
 
-        for name, default in self.defaults.items():
-            value = getattr(settings, prefix + name, default)
+        for name, default in defaults.items():
+            value = getattr(django.conf.settings, prefix + name, default)
             setattr(self, name, value)
