@@ -1,3 +1,4 @@
+from django.conf import settings
 from django.db import models
 
 
@@ -26,6 +27,6 @@ class TestProfile(models.Model):
     A user profile model for use by unit tests. This has nothing to do with the
     authentication backend itself.
     """
-    user = models.OneToOneField('auth.User')
+    user = models.OneToOneField(settings.AUTH_USER_MODEL)
     is_special = models.BooleanField(default=False)
     populated = models.BooleanField(default=False)
